'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS boot program:   _OS.BIN          |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS boot program sets up Upper HUB Memory according to the hardware,
''   then calls the the version module (_VER.CMD),
''   which then calls the command module (_CMD.CMD)
''  
'' The command module issues the prompt "SD:>" and waits for a command,
''   then executes the command internally, or externally or by calling
''   an external module (_xxxxxxx.CMD) or a binary file (xxxxxxxx.BIN)
''
'' All external command modules will return to _CMD.CMD after execution.
''
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
''      Sphinx    (c) 2009-10 by Michael Park and others
'' Some concepts and code is from Sphinx (Michael Park) & Spinix (Dave Hein).
'' 
'' Special thanks to the following authors for their ideas and MIT code:
''   Michael Park, Dave Hein, Kye, James Moxham, Mike Green, Tomas Rokicki,
''   lonesock, hippy, Jeff Ledger, Roger Williams, Michelli Scales, Andy Schenk.
''   My apologies to any others I may have missed.
'' And of course to Chip Gracey for the fabulous Prop chip and the Spin Interpreter.
'' +--------------------------------------------------------------------------+
'' Modifications by Cluso99...
'' RR20120306   000   Initial module _DOS.CMD
'' RR20120323   004   v0.40 "OS.BIN" 
'' RR20120325   005   v0.50 revised hub layout
'' RR20120619   006   v0.60 use separate _SD_PASM.spin & _SD_FAT.spin
''

CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef3"            'fixed hub definitions ($7FFF... down)

  fat   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  taf   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  str   : "ASCII0_STREngine.spin"
  std   : "_PCSIO.spin"                 ' pc i/o driver 
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

dat
csum long   $89                         ' use this to make hub checksum byte @ $0005 = $00 (set to binary byte)

PUB Start | _SDpins, _SIOpins, _SIObaud, cog, z,z0,z1,z2,z3,z4 


  waitcnt(clkfreq * 2 + cnt)                            ' delay 


{------------------------------------------------------------------------------}
'Determine what hardware we are running on
' preset for defined configuration in _hwdef
  _SIOpins := _hw#_SIOpins
  _SIObaud := _hw#_SIObaud
  _SDpins  := _hw#_SDpins                               
' now check for RamBlade/RamBlade3/TriBlade#2
  z := ina                                              'read the input pins (to check pullups)
  if     (z & _hw#_tb2PU) == _HW#_tb2PU                 'TriBlade#2 ?
    _SDpins  := _hw#_tb2
  elseif (z & _hw#_rb1PU) == _hw#_rb1PU                 'RamBlade ?
    _SIOpins := _hw#_sioRB1                             ' serial pins differ on RB1
    _SDpins := _hw#_rb1
  elseif (z & _hw#_rb3PU) == _hw#_rb3PU                 'RamBlade3 ?
    _SDpins := _hw#_rb3
{------------------------------------------------------------------------------}
'Set hub parameters...
  long[_hub#_HubFree]     := _hub#_HUB_RESERVED         ' set top of hub free
  byte[_hub#_OS_Cogs]     := 0                          ' set resident cogs (not to be stopped)
  byte[_hub#_OS_Clkmode]  := byte[$0004]                ' save the current clockmode
  byte[_hub#_OS_Rows]     := _hw#_SCREENROWS | 0<<7     ' Screen rows & <lf> off (_hub#_OS_LFmask)
  byte[_hub#_OS_Columns]  := _hw#_SCREENCOLS            ' Screen columns
  long[_hub#_OS_Clkfreq]  := long[$0000]                ' save the current clockfreq
' long[_hub#_OS_Seconds]  := 126_230_400 * 3            ' Yr2012 = 4yrs*3
  long[_hub#_OS_DateTime] := 0                          '?????
  long[_hub#_SDpins]      := _SDpins & $1F1F1F1F        
  long[_hub#_SIOpins]     := $0F << 24 | _SIOpins       ' no cog running yet
  long[_hub#_SIObaud]     := _SIObaud                              
  long[_hub#_Hardware]    := 1 << 16 | _SDpins >> 29    ' $0001_000x = cluso's hardware
  str.copy(string("ver"),_hub#_pBuffer_A)               ' parameters passed between modules
{------------------------------------------------------------------------------}
'Start serial...
'                 (inRENDEZVOUS, outRENDEZVOUS, serPins,  baud    )
  cog := std.start(_hub#_StdIn,  _hub#_StdOut,  _SIOpins, _SIObaud) ' start driver, returns cog+1
'             (inRENDEZVOUS )
  stdin.start (_hub#_StdIn  )                           ' start the stub
'             (outRENDEZVOUS)
  stdout.start(_hub#_StdOut )                           ' start the stub
{------------------------------------------------------------------------------}
'Update hub parameters...
  long[_hub#_SIOpins]     := (cog-1) << 24 | _SIOpins
  byte[_hub#_OS_COGS]     := (1 << (cog -1))            ' set resident cog
{------------------------------------------------------------------------------}
'   PrintChar(_clearScreenCharacter)                    ' clear screen
    PrintStringCR(string("Loading Propeller OS..."))
{------------------------------------------------------------------------------}
'Start SD driver...
  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
'  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
'    reboot
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  cog := fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1)
  ifnot (cog)
    reboot


  PrintString(string("FAT PASM Cog = "))
  PrintChar((cog -1) | $30)
  CRLF
{------------------------------------------------------------------------------}
'Update hub parameters...
  byte[_hub#_OS_COGS]     |= (1 << (cog -1))            ' set resident cog
{------------------------------------------------------------------------------}
'Access the SD card...
  fat.mountPartition(0)                                 ' mount the sd card
  ifnot(fat.partitionMounted)
    PrintStringCR(string("ERROR: Card Not Mounted, stop!"))
    repeat                                              ' LOOP HERE!!!


  waitcnt(clkfreq * 2 + cnt)                            ' delay 



{------------------------------------------------------------------------------}
'Load/Run "_VER.CMD"...
  waitcnt(clkfreq / 10 + cnt)                           ' delay 
  \str.copy(string("ver"),_hub#_pBuffer_A)              ' calling parameters          
  \fat.bootPartition(string("_VER.CMD"))                ' load/run
{------------------------------------------------------------------------------}
'Only here on error...
  PrintStringCR(string("ERROR: Rebooting..."))
  waitcnt(clkfreq * 2 + cnt)                            ' delay 
  reboot
{------------------------------------------------------------------------------}

PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_Rows] & _hub#_LF_MASK       
      stdout.out(c)                                     ' send <lf>  to output device

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' cr
    PrintChar(_lineFeedCharacter)                       ' lf (optional)

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}