'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _GET     --> _GET.CMD    |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller
'' Compile all modules using Homespun and the OS.BAT file.                              
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
'' See the included code for history and acknowledgments.
''
'' RR20130126   This module does file transfers...
''                  GET <filename.ext> from prop sd card to PC
'' RR20130202   working but not all types of errors catered for! (extra unknown lf char skipped at chksum)



'#include "__MODULE.spin"       ' include the common code for OS modules (PreSpin)
#include "__MODULE.spin"        


CON
 commsize       = 256 '1024     ' default block size for comms
 
DAT
_ModuleStr    byte      "=== GET ===",0       'module's name (string)
 
PRI executeCommand(stringPointer) | fsize, blksize, ch, chksum, csl, csh, c5, i
'' GET command...

    'there should be an <lf> still in the input buffer, so check and clear it
    i := PeekByte
    if i <> 0
      ch := GetByte
      printbyte(ch)

    ' check the file exists                              need to do error checking!!!==========
    fat.openFile(str.tokenizeString(0), "R")
    fsize := fat.fileSize                               ' file size
    blksize := commsize                                 ' default blocksize
    if fsize>0 
      printbyte("A")                                    ' ACK
    else
      printbyte("T")                                    ' Terminate
      abort -2

    ' send blocks
    repeat
      if fsize =< blksize                               ' if less than blocksize,
        blksize := fsize                                '   adjust bocksize to bytes remaining
      fsize -= blksize                                  ' reduce remaining size (can be 0)
      ' send blocksize (BL,BH)
      printbyte(blksize & $FF)                          ' send blocksize Lo (last block will be =0)
      printbyte((blksize >> 8) & $FF)                   '                Hi
      chksum~
      if blksize > 0                                    ' bytes to be sent
        ' send bytes
        repeat blksize
          ch := (fat.readByte)                          ' read byte
          printbyte(ch)                                 ' send byte
          chksum += ch                                  ' accum checksum
      ' wait for checksum CL & CH
      ch := GetByte                                     ' recv checksum Lo
      ch := ch + (GetByte << 8)                         '               Hi
'''      ch := chksum                          '!!!======== force checksum equal
      if (chksum & $FFFF) <> (ch & $FFFF)               ' verify checksum (last block will be =0)
        printbyte("T")                                  ' terminate
        printbyte(ch & $FF)
        printbyte((ch>>8) & $FF)
        printbyte("<")
        printbyte(">")
        printbyte(chksum & $FF)
        printbyte((chksum>>8) & $FF)
        fat.closeFile
        abort -1

{{
     csl := GetByte
      csh := Getbyte
      printbyte("~")
      printbyte(csl)
      printbyte(csh)
      printbyte(chksum & $FF)
      printbyte((chksum>>8) & $FF)
      chksum -= (csl + (csh<<8))
      printbyte(chksum & $FF)
      printbyte((chksum>>8) & $FF)
      if chksum == 0
        printbyte("=")
      else
        printbyte(">")
}}
{{
      printbyte("{")
      repeat
        c5 := GetByte
        printbyte(c5)
}}
{{
'''''      ch := chksum                     '!!!             
      if (chksum & $FFFF) <> (ch & $FFFF)               ' verify checksum (last block will be =0)
        printbyte("T")                                  ' terminate
        printbyte(ch & $FF)
        printbyte((ch>>8) & $FF)
        printbyte(chksum & $FF)
        printbyte((chksum>>8) & $FF)
        fat.closeFile
        abort -1
}}
    until blksize == 0                                  ' (fat.fileSize == fat.fileTell)
    fat.closeFile
    printbyte("A")                                      ' ACK = success
'wait until we get the END from the pc
    repeat
      ch := GetByte
      if ch <> "E"
        printbyte(ch)
    until ch == "E"  
  ' all done so finish up
    abort 0

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}